@@ -1,4 +1,5 @@
 from shapely.geometry.base import BaseGeometry
+from shapely.geometry import box, Point, Polygon, MultiPolygon
 import shapely.wkt
 import geopandas as gpd
 import pandas as pd
@@ -6,6 +7,21 @@ from sqlalchemy import create_engine
 from functools import lru_cache, partial
 import s2sphere
 
+from mundipy.cache import spatial_cache_footprint
+
+r = s2sphere.RegionCoverer()
+
+def tile_bbox(polygon):
+	bbox = polygon.bounds
+
+	p1 = s2sphere.LatLng.from_degrees(bbox[3], bbox[0])
+	p2 = s2sphere.LatLng.from_degrees(bbox[1], bbox[2])
+
+	rect = s2sphere.LatLngRect.from_point_pair(p1, p2)
+
+	# 14 is ~0.3km2 and 600m edge length
+	return r.get_simple_covering(rect, p1.to_point(), 14)
+
 """A Layer represents a group of spatial data."""
 class Layer:
 
@@ -28,10 +44,18 @@ class Layer:
 		else:
 			raise TypeError('data for Layer() is not filename, GeoDataFrame or dict with PostgreSQL details')
 
-	def _load(self, bbox=None, pcs='EPSG:4326'):
-		"""Load part or the entire Layer as a dataframe."""
+	@spatial_cache_footprint
+	def _load(self, bbox, pcs='EPSG:4326'):
+		"""
+		Load part or the entire Layer as a dataframe.
+
+		Takes bbox as a 4-tuple of WGS84 coordinates (lon, lat). bbox can be
+		None to load the entire dataset.
+
+		Returns the dataset in PCS coordinates.
+		"""
 		if self._dataframe is not None:
-			return self._dataframe
+			return (self._dataframe, None)
 
 		if self._db_url is not None:
 			# load from PostGIS
@@ -40,22 +64,13 @@ class Layer:
 			# no bbox
 			if bbox is None:
 				# build the query
-				query = "SELECT *, ST_Transform(geometry, '%s') AS local_geom FROM %s" % (pcs, self._db_table)
+				query = "SELECT * FROM %s" % self._db_table
 
-				gdf = gpd.GeoDataFrame.from_postgis(query, self._conn, geom_col='local_geom', crs=pcs)
-				gdf.set_geometry('local_geom', inplace=True)
+				gdf = gpd.GeoDataFrame.from_postgis(query, self._conn, geom_col='geometry', crs='EPSG:4326')
 				self._conn = None
-				return gdf
+				return (gdf.to_crs(pcs), None)
 
-			# tile the area
-			r = s2sphere.RegionCoverer()
-			# cell level 20 = typical edge length of 7-10m or 30ft
-			r.max_level = 20
-
-			p1 = s2sphere.LatLng.from_degrees(bbox[3], bbox[0])
-			p2 = s2sphere.LatLng.from_degrees(bbox[1], bbox[2])
-
-			cell_ids = r.get_covering(s2sphere.LatLngRect.from_point_pair(p1, p2))
+			cell_ids = list(tile_bbox(bbox))
 
 			tiles = [self._load_tile(cellid, pcs) for cellid in cell_ids]
 			together = pd.concat(tiles)
@@ -63,12 +78,17 @@ class Layer:
 
 			self._conn = None
 
-			return together_geo
+			# build footprint from cell_ids
+			# needs 5 points to have a box polygon
+			polygons = [[s2sphere.LatLng.from_point(s2sphere.Cell(cellid).get_vertex(v)) for v in [0, 1, 2, 3, 0]] for cellid in cell_ids]
+			polygons = [Polygon([Point(v.lng().degrees, v.lat().degrees) for v in polygon]) for polygon in polygons]
+
+			return (together_geo, MultiPolygon(polygons).buffer(0))
 
 		if bbox is None:
-			return gpd.read_file(self.filename)
+			return (gpd.read_file(self.filename), None)
 		else:
-			return gpd.read_file(self.filename, bbox=bbox)
+			return (gpd.read_file(self.filename, bbox=bbox), None)
 
 	@lru_cache(maxsize=512)
 	def _load_tile(self, cellid, pcs):
@@ -83,23 +103,22 @@ class Layer:
 		)
 		# shapely.wkt.loads(wkt)
 
-		query = "SELECT *, ST_Transform(geometry, '%s') AS local_geom FROM %s WHERE geometry && ST_GeomFromEWKT('SRID=4326;%s')" % (pcs, self._db_table, wkt)
+		query = "SELECT * FROM %s WHERE geometry && ST_GeomFromEWKT('SRID=4326;%s')" % (self._db_table, wkt)
 
-		gdf = gpd.GeoDataFrame.from_postgis(query, self._conn, geom_col='local_geom', crs=pcs)
-		gdf.set_geometry('local_geom', inplace=True)
-		return gdf
+		gdf = gpd.GeoDataFrame.from_postgis(query, self._conn, geom_col='geometry', crs='EPSG:4326')
+		return gdf.to_crs(pcs)
 
 	@property
-	def dataframe(self, pcs='EPSG:4326'):
+	def dataframe(self):
 		"""Load an entire Layer as a dataframe."""
-		return self._load(pcs=pcs)
+		return self._load(None)
 
 	"""Read into a Layer at a specific geometry (WGS84)."""
 	def inside_bbox(self, bbox, pcs='EPSG:4326'):
 		if not isinstance(bbox, tuple) or len(bbox) != 4:
 			raise TypeError('inside_bbox expected bbox to be a 4-tuple')
 
-		return self._load(bbox=bbox, pcs=pcs)
+		return self._load(box(*bbox), pcs=pcs)
 
 
 """VisibleLayer represents Layer data, as seen from a geometry."""
@@ -123,11 +142,14 @@ class VisibleLayer:
 
 	def intersects(self):
 		"""Give a list of members that intersect with the center geometry."""
-		spatial_index = self.local_collection.sindex
-		possible_matches_index = list(spatial_index.intersection(self.center.bounds))
-		possible_matches = self.local_collection.iloc[possible_matches_index]
+		possible_matches_index = list(self.local_collection.sindex.intersection(self.center.bounds))
 
-		return possible_matches[possible_matches.intersects(self.center)]
+		# indexing into dataframes is expensive, compute a new list of actual matches
+		# .iloc only takes integer locations, even for columns
+		geometry_col_loc = self.local_collection.columns.get_loc('geometry')
+		actual_matches_index = [match for match in possible_matches_index if self.local_collection.iloc[match, geometry_col_loc].intersects(self.center)]
+
+		return self.local_collection.iloc[actual_matches_index]
 
 	def nearest(self):
 		"""Give the closest member to the center geometry."""
