@@ -16,8 +16,11 @@ from shapely.geometry import shape
 from shapely.geometry.base import BaseGeometry
 import fiona
 
-CRS_DATASET = os.path.join(os.path.dirname(__file__), 'crs_simple.fgb')
+from mundipy.cache import spatial_cache_footprint
 
+CRS_DATASET = os.path.join(os.path.dirname(__file__), 'crs_enhanced.fgb')
+
+@spatial_cache_footprint
 def choose_pcs(box, units='meters'):
 	"""Choose a projected coordinate system from a shapely geometry with specified units for the axes."""
 
@@ -26,11 +29,24 @@ def choose_pcs(box, units='meters'):
 	if units not in ['feet', 'meters']:
 		raise TypeError('units passed to projectedcrs.choose_pcs() is neither feet nor meters')
 
-	suggestions = suggest_pcs(box, units=units)
-	if len(suggestions) == 0:
-		raise ValueError('choose_pcs(box) had zero suggested CRS (hint: is your box in WGS84 format?)')
+	with fiona.open(CRS_DATASET, 'r') as epsg:
+		intersects_box = epsg.values(bbox=box.bounds)
 
-	return suggestions[0]
+		# filter for proprety
+		has_unit = filter(lambda geo: geo['geometry'] is not None and geo['properties']['axis_unit'] == units, intersects_box)
+
+		# sort by area
+		by_area = sorted(has_unit, key=lambda geo: geo['properties']['area'])
+
+		# incrementally make list of suggestions, adding if contains
+		suggestions = []
+
+		for potential_pcs in by_area:
+			geo = shape(potential_pcs['geometry']).buffer(0)
+			if not geo.contains(box):
+				continue
+
+			return (transform_epsg(potential_pcs), geo)
 
 def suggest_pcs(box, units='meters', n=3):
 	"""Suggest multiple projected coordinate systems from a shapely geometry with specified units for the axes."""
@@ -41,18 +57,27 @@ def suggest_pcs(box, units='meters', n=3):
 		raise TypeError('units passed to projectedcrs.suggest_pcs() is neither feet nor meters')
 
 	with fiona.open(CRS_DATASET, 'r') as epsg:
-		intersects_box = [geo for i, geo in epsg.items(bbox=box.bounds) if geo['geometry'] is not None]
+		intersects_box = epsg.values(bbox=box.bounds)
 
 		# filter for proprety
-		has_unit = filter(lambda geo: geo['properties']['axis_unit'] == units, intersects_box)
-
-		# filter for where items contain box
-		contains_box = filter(lambda geo: shape(geo['geometry']).buffer(0).contains(box), has_unit)
+		has_unit = filter(lambda geo: geo['geometry'] is not None and geo['properties']['axis_unit'] == units, intersects_box)
 
 		# sort by area
-		satisfactory = list(map(transform_epsg, sorted(contains_box, key=lambda geo: shape(geo['geometry']).area)))
+		by_area = sorted(has_unit, key=lambda geo: geo['properties']['area'])
 
-		return satisfactory[:n]
+		# incrementally make list of suggestions, adding if contains
+		suggestions = []
+
+		for potential_pcs in by_area:
+			if not shape(potential_pcs['geometry']).buffer(0).contains(box):
+				continue
+
+			suggestions.append(potential_pcs)
+
+			if len(suggestions) >= n:
+				break
+
+		return list(map(transform_epsg, suggestions))
 
 def transform_epsg(geojson):
 	return {
